'****************************************************************
'*  Name    : key15-t.BAS                                       *
'*  Author  : Zuisti                                            *
'*  Notice  : Copyright (c) 2011                                *
'*          : All Rights Reserved                               *
'*  Date    : 2011. 03. 29                                      *
'*  Version : 2.0                                               *
'*  Notes   : debouncing via timer2 IT (20 mS), examine release *
'****************************************************************

    Device = 16F628A 
    Xtal = 4
    Optimiser_Level = 1

    Dim key             As Byte 'pressed key 
    Dim KeyReads        As Byte 'Portb upper nybble (keys) 00..F0     
    Dim cnt             As Byte 'key release counter

    Dim IT_Flag         As Bit

    Symbol RBIF = INTCON.0  'key changed it flag
    Symbol TMR2IF = PIR1.1  'timer2 it flag
    Symbol TMR2ON = T2CON.2 'timer2 on/off


    Declare FSR_CONTEXT_SAVE = OFF
    On_Hardware_Interrupt GoTo KeyIT

    Call Hw_Init
    GoTo Start

Look_Key:
    'addwf WREG, w 'PIC18 only!
    addwf PCL ; go to the proper 'retlw xx' instr, returns  with result in wreg
        retlw 0     ;0 not used
        
        retlw "1"   ;0001  1
        retlw "2"   ;0010  2
        retlw "3"   ;0011  3
        retlw "4"   ;0100  4
        retlw "5"   ;0101  5
        
        retlw "6"   ;0110  6
        retlw "7"   ;0111  7  (use value zero if a key not used)
        retlw "8"   ;1000  8
        retlw "9"   ;1001  9
        retlw "0"   ;1010 10
        
        retlw "a"   ;1011 11
        retlw "b"   ;1100 12
        retlw "c"   ;1101 13
        retlw "d"   ;1110 14
        retlw "e"   ;1111 15

KeyIT:
    If TMR2IF = 1 Then          ' timer2 it?
        Clear TMR2IF            ' clear timer2 it flag
        Clear TMR2ON            ' timer2 off
        movf PORTB, w           ' now get key pins again
        andlw 0xF0              ' upper nybble only
        xorlw 0xF0              ' invert
        If STATUS.2 = 1 Then    ' zero?
            clrf KeyReads       ' if w (new keys) was 0 then KeyReads = 0 
        ElseIf KeyReads = 0 Then' do not accept new key until release all
            movwf KeyReads      ' save it (using for key repeat, later)      
            swapf KeyReads,w    ' transform it to 0...15  
            Call Look_Key       ' get the key code from table (to w)
            andlw 255           ' drop zero key value (not used)
            If STATUS.2 = 0 Then' nonzero?
                If IT_Flag = 0 Then 
                    movwf key
                    Set IT_Flag
                End If
            EndIf
        EndIf
        While 1 = 1
            movf PORTB, w       ' clear mismatch condition again
            nop
            Clear RBIF          ' clear it flag
            nop
            btfsc RBIF          ' success? 
        Wend        
        Set INTCON.3            ' RBIE, enable portb change pin it    
    ElseIf RBIF = 1 Then        ' pin change it?
        Clear INTCON.3          ' RBIE, disable portb change pin it
'        While 1 = 1
'            movf PORTB, w 
'            nop
'            Clear RBIF 
'            nop
'            btfsc RBIF
'        Wend
        Set TMR2ON              ' timer2 on (20 mS debouncing)
    EndIf
  Context Restore ' Restore regs and return from interrupt 

' ***************** Program starts here

Hw_Init:
    All_Digital = TRUE
    TRISB = 0xF0            ' portb.4..7 are inputs for key read
    PORTB = 0xFF            ' init portb
    Clear OPTION_REG.7      ' RBPU, portb pullups on        		
    Clear INTCON            ' Disable Interrupts (all)
    Clear IT_Flag
    Clear KeyReads
    movf PORTB, w           ' clear mismatch condition
    Clear cnt 'for release
    Clear RBIF              ' clear key change it flag
    Set INTCON.3            ' RBIE, enable portb change pin it

    'Timer2 Prescaler= 16, PostScaler = 5, PR2 = 250
    'Freq = 50.00 Hz - Period = 0.020000 seconds
    'T2CON |= 32;        // bits 6-3 Post scaler 1:1 thru 1:16
    'T2CON.TMR2ON = 1;  // bit 2 turn timer2 on;
    'T2CON.T2CKPS1 = 1; // bits 1-0  Prescaler Rate Select bits
    'T2CON.T2CKPS0 = 0;
    T2CON = 32 + 2
    PR2 = 250               ' PR2 (Timer2 Match value)
    Clear TMR2IF            ' timer2 it flag
    Set PIE1.1              ' TMR2IE = 1;enable Timer2 interrupts
    'Set TMR2ON             ' timer2 NOT on here !!!!

    Set INTCON.6            ' PEIE = 1;bit6 Peripheral Interrupt Enable bit...1 = Enables all unmasked peripheral interrupts
    Set INTCON.7            ' GIE, enable global IT
 Return ' Hw_init

Start:
    ' for demonstrating only (:-)
    HSerOut ["15 keys on 4 pins (made by zuisti).",13,13]
    'using default hserout settings (2400 Bd at 4 MHz)
    
'-- release ---------------------------------------------
    Set TMR2ON 
    DelayMS 25
    If IT_Flag = 1 Then
        Clear IT_Flag
        HSerOut ["* Pressed key(s) found at power on !",13,_
                 "* Waiting for release all keys ..."]
        While KeyReads <> 0
            DelayMS 25        
            Inc cnt
        Wend
        HSerOut [" Thanks !",13]
    End If
'-- release ---------------------------------------------
    
    HSerOut ["New key pressed:",13]

    While 1=1 'main forever loop

        If IT_Flag = 1 Then
            HSerOut [key,";"]       
            Clear IT_Flag
        EndIf

'-- release ---------------------------------------------
        Clear cnt
        While KeyReads <> 0
            DelayMS 25        
            Inc cnt
            If cnt = 20 Then 'after 20x25 = 500 msec
                HSerOut [13,"* Waiting for release key(s) ..."]
                While KeyReads <> 0
                    DelayMS 25        
                    Inc cnt
                Wend
                HSerOut [" Thanks !",13,"New key pressed:",13]            
            EndIf
        Wend
'-- release ---------------------------------------------
        
    Wend 'main forever loop

'end of program
    
